#include <stdio.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "BME280.h"
#include "extern.h"

void BME280_init(void)
{
    // I2C Initialisation. Using it at 400kHz.
    i2c_init(BME280_I2C_PORT, 50 * 1000);
    gpio_set_function(BME280_I2C_SDA, GPIO_FUNC_I2C);
    gpio_set_function(BME280_I2C_SCL, GPIO_FUNC_I2C);
    gpio_pull_up(BME280_I2C_SDA);
    gpio_pull_up(BME280_I2C_SCL);

    uint8_t control_humidity[2] = {0};
    control_humidity[0] = BME280_CTRL_HUMIDITY_REG;
    control_humidity[1] = 0b00000101; // x16
    i2c_write_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, control_humidity, sizeof(control_humidity), false);

    uint8_t control_measure[2] = {0};
    control_measure[0] = BME280_CTRL_MEAS_REG;
    control_measure[1] = 0b10110111; // x16 x16 normal mode
    i2c_write_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, control_measure, sizeof(control_measure), false);

    uint8_t config[2] = {0};
    config[0] = BME280_CONFIG_REG;
    config[1] = 0b10010000; // 500ms 16filter
    i2c_write_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, config, sizeof(config), false);

    uint8_t param[25] = {0};
    uint8_t reg = BME280_DIG_T1_LSB_REG;
    i2c_write_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, &reg, 1, true);
    i2c_read_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, param, sizeof(param), false);
    BME280_param.dig_T1 = param[0] | (param[1] << 8);
    BME280_param.dig_T2 = param[2] | (param[3] << 8);
    BME280_param.dig_T3 = param[4] | (param[5] << 8);
    BME280_param.dig_P1 = param[6] | (param[7] << 8);
    BME280_param.dig_P2 = param[8] | (param[9] << 8);
    BME280_param.dig_P3 = param[10] | (param[11] << 8);
    BME280_param.dig_P4 = param[12] | (param[13] << 8);
    BME280_param.dig_P5 = param[14] | (param[15] << 8);
    BME280_param.dig_P6 = param[16] | (param[17] << 8);
    BME280_param.dig_P7 = param[18] | (param[19] << 8);
    BME280_param.dig_P8 = param[20] | (param[21] << 8);
    BME280_param.dig_P9 = param[22] | (param[23] << 8);
    BME280_param.dig_H1 = param[24];
    uint8_t param2[7] = {0};
    reg = BME280_DIG_H2_LSB_REG;
    i2c_write_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, &reg, 1, true);
    i2c_read_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, param2, sizeof(param2), false);
    BME280_param.dig_H2 = param2[0] | (param2[1] << 8);
    BME280_param.dig_H3 = param2[2];
    BME280_param.dig_H4 = param2[3] | (param2[4] << 8);
    BME280_param.dig_H5 = param2[4] | (param2[5] << 8);
    BME280_param.dig_H6 = param2[6];
}

void BME280_read_all(void)
{
    uint8_t buf[8] = {0};
    uint8_t reg = BME280_PRESSURE_MSB_REG;
    i2c_write_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, &reg, 1, true);
    i2c_read_blocking(BME280_I2C_PORT, BME280_DEVICE_ADDR, buf, sizeof(buf), false);

    uint32_t pressure_raw, temparature_raw, humidity_raw;
    pressure_raw = ((uint32_t)buf[0] << 12) | ((uint32_t)buf[1] << 4) | (buf[2] >> 4);
    temparature_raw = ((uint32_t)buf[3] << 12) | ((uint32_t)buf[4] << 4) | (buf[5] >> 4);
    humidity_raw = ((uint32_t)buf[6] << 8) | buf[7];

    // temparature
    int32_t temp_var1, temp_var2, T, t_fine;
    temp_var1 = ((((temparature_raw >> 3) - ((int32_t)BME280_param.dig_T1 << 1))) * ((int32_t)BME280_param.dig_T2)) >> 11;
    temp_var2 = (((((temparature_raw >> 4) - ((int32_t)BME280_param.dig_T1)) * ((temparature_raw >> 4) - ((int32_t)BME280_param.dig_T1))) >> 12) * ((int32_t)BME280_param.dig_T3)) >> 14;
    t_fine = (temp_var1 + temp_var2);
    T = (t_fine * 5 + 128) >> 8;
    temparature = T;

    // pressure
    int32_t press_var1, press_var2, P;
    press_var1 = (((int32_t)t_fine) >> 1) - (int32_t)64000;
    press_var2 = (((press_var1 >> 2) * (press_var1 >> 2)) >> 11) * ((int32_t)BME280_param.dig_P6);
    press_var2 = press_var2 + ((press_var1 * ((int32_t)BME280_param.dig_P5)) << 1);
    press_var2 = (press_var2 >> 2) + (((int32_t)BME280_param.dig_P4) << 16);
    press_var1 = (((BME280_param.dig_P3 * (((press_var1 >> 2) * (press_var1 >> 2)) >> 13)) >> 3) + ((((int32_t)BME280_param.dig_P2) * press_var1) >> 1)) >> 18;
    press_var1 = ((((32768 + press_var1)) * ((int32_t)BME280_param.dig_P1)) >> 15);
    if (press_var1 == 0)
    {
        P = 0;
    }
    else
    {
        P = (((uint32_t)(((int32_t)1048576) - pressure_raw) - (press_var2 >> 12))) * 3125;
        if (P < 0x80000000)
        {
            P = (P << 1) / ((uint32_t)press_var1);
        }
        else
        {
            P = (P / (uint32_t)press_var1) * 2;
        }
        press_var1 = (((int32_t)BME280_param.dig_P9) * ((int32_t)(((P >> 3) * (P >> 3)) >> 13))) >> 12;
        press_var2 = (((int32_t)(P >> 2)) * ((int32_t)BME280_param.dig_P8)) >> 13;
        P = (uint32_t)((int32_t)P + ((press_var1 + press_var2 + BME280_param.dig_P7) >> 4));
        pressure = P;
    }

    // humidity
    uint32_t v_x1_u32r, H;
    v_x1_u32r = (t_fine - ((int32_t)76800));
    v_x1_u32r = (((((humidity_raw << 14) - (((int32_t)BME280_param.dig_H4) << 20) - (((int32_t)BME280_param.dig_H5) * v_x1_u32r)) + ((int32_t)16384)) >> 15) * (((((((v_x1_u32r * ((int32_t)BME280_param.dig_H6)) >> 10) * (((v_x1_u32r * ((int32_t)BME280_param.dig_H3)) >> 11) + ((int32_t)32768))) >> 10) + ((int32_t)2097152)) * ((int32_t)BME280_param.dig_H2) + 8192) >> 14));
    v_x1_u32r = (v_x1_u32r - (((((v_x1_u32r >> 15) * (v_x1_u32r >> 15)) >> 7) * ((int32_t)BME280_param.dig_H1)) >> 4));
    v_x1_u32r = (v_x1_u32r < 0 ? 0 : v_x1_u32r);
    v_x1_u32r = (v_x1_u32r > 419430400 ? 419430400 : v_x1_u32r);
    H = (v_x1_u32r >> 12);
    humidity = H;

#ifndef NDEBUG
    printf("T=%8.2lf, P=%8.3lf, H=%8.4lf\n", T / (float)100.0, (float)P / 100.0, (float)H / 1024.0);
#endif
}