#ifndef __DS3231_H__
#define __DS3231_H__

#include <stdint.h>
#include "pico/stdlib.h"

#define TIME_ZONE (+9)
#define TIME_ZONE_MAX (+14)
#define TIME_ZONE_MIN (-12)

void DS3231_init(struct repeating_timer *timer1, struct repeating_timer *timer2);
void DS3231_read_all(void);
void DS3231_read_sec(void);
void DS3231_write_time(uint8_t sec1, uint8_t sec2, uint8_t min1, uint8_t min2, uint8_t hour1, uint8_t hour2);
void DS3231_write_time_decimal(uint8_t hour, uint8_t min, uint8_t sec);
void DS3231_write_date(uint8_t date1, uint8_t date2, uint8_t month1, uint8_t month2, uint8_t year1, uint8_t year2);
void DS3231_write_date_decimal(uint8_t year, uint8_t mon, uint8_t date);

#define I2C_PORT i2c0
#define DS3231_DEVICE_ADDR 0x68
#define I2C_SDA 0
#define I2C_SCL 1
#define RTC_1PPS_PIN 2
#define RTC_RESET_PIN 3

struct DS3231
{
    /* data */
    uint8_t sec, min, hour, day, date, mon, year;
    uint8_t AL1_sec, AL1_min, AL1_hour, AL1_date;
    uint8_t AL2_min, AL2_hour, AL2_date;
    uint8_t control, status;
    uint8_t ageing_offset;
    uint8_t temp_MSB, temp_LSB;
};

// DS3231 timekeeping registers
#define DS3231_SEC_ADDR            0x00
#define DS3231_MIN_ADDR            0x01
#define DS3231_HOUR_ADDR           0x02
#define DS3231_DAY_ADDR            0x03
#define DS3231_DATE_ADDR           0x04
#define DS3231_MONTH_ADDR          0x05
#define DS3231_YEAR_ADDR           0x06
#define DS3231_ALARM1_SEC_ADDR     0x07
#define DS3231_ALARM1_MIN_ADDR     0x08
#define DS3231_ALARM1_HOUR_ADDR    0x09
#define DS3231_ALARM1_DATE_ADDR    0x0A
#define DS3231_ALARM2_MIN_ADDR     0x0B
#define DS3231_ALARM2_HOUR_ADDR    0x0C
#define DS3231_ALARM2_DATE_ADDR    0x0D
#define DS3231_CONTROL_ADDR        0x0E
#define DS3231_STATUS_ADDR         0x0F
#define DS3231_AGING_OFFSET_ADDR   0x10
#define DS3231_TEMP_MSB_ADDR       0x11 // raed only
#define DS3231_TEMP_LSB_ADDR       0x12 // read only

// DS3231 control register map
#define DS3231_CONTROL_EOSC  0b10000000
#define DS3231_CONTROL_BBSQW 0b01000000
#define DS3231_CONTROL_CONV  0b00100000
//#define DS3231_CONTROL_RS2   0b00010000
//#define DS3231_CONTROL_RS1   0b00001000
#define DS3231_CONTROL_INTCN 0b00000100
#define DS3231_CONTROL_A2IE  0b00000010
#define DS3231_CONTROL_A1IE  0b00000001

// DS3231 status register map
#define DS3231_STATUS_OSF    0b10000000
//#define DS3231_STATUS_BB32   0b01000000
//#define DS3231_STATUS_CRATE1 0b00100000
//#define DS3231_STATUS_CRATE2 0b00010000
#define DS3231_STATUS_EN32   0b00001000
#define DS3231_STATUS_BSY    0b00000100
#define DS3231_STATUS_A2F    0b00000010
#define DS3231_STATUS_A1F    0b00000001

#endif