#include <stdio.h>
#include <stdlib.h>
#include "hardware/spi.h"
#include "hardware/pwm.h"
#include "pico/stdlib.h"
#include "common.h"
#include "DS3231.h"
#include "Nixie_Driver.h"
#include "shuffle.h"
#include "extern.h"

// uint slice_num_LD = 0;
// uint slice_num_RD = 0;

void SPI_init(struct repeating_timer *timer)
{
    // SPI initialisation. This example will use SPI at 50MHz.
    spi_init(SPI_PORT, 50 * 1000 * 1000);
    gpio_set_function(PIN_MISO, GPIO_FUNC_SPI);
    gpio_set_function(PIN_CS, GPIO_FUNC_SIO);
    gpio_set_function(PIN_SCK, GPIO_FUNC_SPI);
    gpio_set_function(PIN_MOSI, GPIO_FUNC_SPI);
    // Chip select is active-low, so we'll initialise it to a driven-high state
    gpio_init(PIN_CS);
    gpio_set_dir(PIN_CS, GPIO_OUT);
    gpio_put(PIN_CS, true);

    // Dot PIN init
    nixie_dot_init();
    // PWM
    // nixie_dot_pwm_init();

    // attach driver callback
    add_repeating_timer_us(-10, alarm_nixie_driver_callback, NULL, timer);
}

static inline void cs_select(void)
{
    asm volatile("nop \n nop \n nop");
    gpio_put(PIN_CS, false); // Active low
    asm volatile("nop \n nop \n nop");
}

static inline void cs_deselect(void)
{
    asm volatile("nop \n nop \n nop");
    gpio_put(PIN_CS, true);
    asm volatile("nop \n nop \n nop");
}

void nixie_dot_init(void)
{
    gpio_init(NIXIE_LD_PIN);
    gpio_init(NIXIE_RD_PIN);
    gpio_set_dir(NIXIE_LD_PIN, GPIO_OUT);
    gpio_set_dir(NIXIE_RD_PIN, GPIO_OUT);
}

/* 
void nixie_set_LD_pwm(int pwm)
{
    pwm_set_chan_level(slice_num_LD, PWM_CHAN_B, pwm);
    pwm_set_enabled(slice_num_LD, true);
}

void nixie_set_RD_pwm(int pwm)
{
    pwm_set_chan_level(slice_num_RD, PWM_CHAN_A, pwm);
    pwm_set_enabled(slice_num_RD, true);
}

void nixie_dot_pwm_init(void)
{
    slice_num_RD = pwm_gpio_to_slice_num(NIXIE_RD_PIN);
    gpio_init(NIXIE_RD_PIN);
    gpio_set_dir(NIXIE_RD_PIN, GPIO_OUT);
    gpio_set_function(NIXIE_RD_PIN, GPIO_FUNC_PWM);
    pwm_set_clkdiv(slice_num_RD, NIXIE_DOT_PWM_CLKDIV);
    pwm_set_wrap(slice_num_RD, NIXIE_DOT_PWM_WRAP);
    pwm_set_chan_level(slice_num_RD, PWM_CHAN_A, 0);
    pwm_set_enabled(slice_num_RD, true);
    slice_num_LD = pwm_gpio_to_slice_num(NIXIE_LD_PIN);
    gpio_init(NIXIE_LD_PIN);
    gpio_set_dir(NIXIE_LD_PIN, GPIO_OUT);
    gpio_set_function(NIXIE_LD_PIN, GPIO_FUNC_PWM);
    pwm_set_clkdiv(slice_num_LD, NIXIE_DOT_PWM_CLKDIV);
    pwm_set_wrap(slice_num_LD, NIXIE_DOT_PWM_WRAP);
    pwm_set_chan_level(slice_num_LD, PWM_CHAN_B, 0);
    pwm_set_enabled(slice_num_LD, true);
}
 */

void dot_pattern(void)
{
    switch (dot_mode)
    {
    case 0: // RD LD blink
        if ((ds3231.sec % 2) == 0)
        {
            gpio_put(NIXIE_LD_PIN, false);
            gpio_put(NIXIE_RD_PIN, true);
            // nixie_set_LD_pwm(0);
            // nixie_set_RD_pwm(NIXIE_DOT_PWM_MAX);
        }
        else
        {
            gpio_put(NIXIE_LD_PIN, true);
            gpio_put(NIXIE_RD_PIN, false);
            // nixie_set_LD_pwm(NIXIE_DOT_PWM_MAX);
            // nixie_set_RD_pwm(0);
        }
        break;

    case 1: // RD ON
        gpio_put(NIXIE_LD_PIN, false);
        gpio_put(NIXIE_RD_PIN, true);
        // nixie_set_LD_pwm(0);
        // nixie_set_RD_pwm(NIXIE_DOT_PWM_MAX);
        break;

    case 2: // LD ON
        gpio_put(NIXIE_LD_PIN, true);
        gpio_put(NIXIE_RD_PIN, false);
        // nixie_set_LD_pwm(NIXIE_DOT_PWM_MAX);
        // nixie_set_RD_pwm(0);
        break;

    case 3: // RD blink
        if ((ds3231.sec % 2) == 0)
        {
            gpio_put(NIXIE_LD_PIN, false);
            gpio_put(NIXIE_RD_PIN, true);
            // nixie_set_LD_pwm(0);
            // nixie_set_RD_pwm(NIXIE_DOT_PWM_MAX);
        }
        else
        {
            gpio_put(NIXIE_LD_PIN, false);
            gpio_put(NIXIE_RD_PIN, false);
            // nixie_set_LD_pwm(0);
            // nixie_set_RD_pwm(0);
        }
        break;

    case 4: // LD blink
        if ((ds3231.sec % 2) == 0)
        {
            gpio_put(NIXIE_LD_PIN, true);
            gpio_put(NIXIE_RD_PIN, false);
            // nixie_set_LD_pwm(NIXIE_DOT_PWM_MAX);
            // nixie_set_RD_pwm(0);
        }
        else
        {
            gpio_put(NIXIE_LD_PIN, false);
            gpio_put(NIXIE_RD_PIN, false);
            // nixie_set_LD_pwm(0);
            // nixie_set_RD_pwm(0);
        }
        break;

    case 5: // OFF
        gpio_put(NIXIE_LD_PIN, false);
        gpio_put(NIXIE_RD_PIN, false);
        // nixie_set_LD_pwm(0);
        // nixie_set_RD_pwm(0);
        break;

    default:
        dot_mode = 0;
        break;
    }
}

void normal_mode(void)
{
    static int duty_count = 0;
    static int tube_pos = 0;
    uint8_t buf[3] = {0, 0, 0}; // SPI write buffer

    if (duty_count < setting.duty) // ON
    {
        if (tube_pos == 2 || tube_pos == 5)
        {
            dot_pattern();
            buf[0] = digit_num[tube_pos][0];
            buf[1] = digit_num[tube_pos][1];
            buf[2] = tube_num[tube_pos];
        }
        else
        {
            if (mode == 0 && setting.use_xfade == true)
            {
                bool do_plus = false;
                if (xfade_count > setting.xfade_length)
                {
                    do_plus = rand_xor_shift() % 100 < (xfade_count - 20) ? 1 : 0; // xfade_count increase 0 - 100 until 1sec
                }
                if (tube_pos == 7)                                           // sec always
                {
                    uint8_t buf_sec = (ds3231.sec & 0b00001111) + do_plus;
                    if (buf_sec == 10)
                        buf_sec = 0;
                    buf[0] = num_order[buf_sec][0];
                    buf[1] = num_order[buf_sec][1];
                }
                else if (tube_pos == 6)
                {
                    if ((ds3231.sec & 0b00001111) == 9) // sec is 9
                    {
                        uint8_t buf_sec = ((ds3231.sec & 0b11110000) >> 4) + do_plus;
                        if (buf_sec == 6)
                            buf_sec = 0;
                        buf[0] = num_order[buf_sec][0];
                        buf[1] = num_order[buf_sec][1];
                    }
                    else
                    {
                        buf[0] = num_order[((ds3231.sec & 0b11110000) >> 4)][0];
                        buf[1] = num_order[((ds3231.sec & 0b11110000) >> 4)][1];
                    }
                }
                else if (tube_pos == 4)
                {
                    if (((ds3231.sec & 0b11110000) >> 4) == 5 && (ds3231.sec & 0b00001111) == 9) // sec is 59
                    {
                        uint8_t buf_min = (ds3231.min & 0b00001111) + do_plus;
                        if (buf_min == 10)
                            buf_min = 0;
                        buf[0] = num_order[buf_min][0];
                        buf[1] = num_order[buf_min][1];
                    }
                    else
                    {
                        buf[0] = num_order[(ds3231.min & 0b00001111)][0];
                        buf[1] = num_order[(ds3231.min & 0b00001111)][1];
                    }
                }
                else if (tube_pos == 3)
                {
                    if ((ds3231.min & 0b00001111) == 9 && ((ds3231.sec & 0b11110000) >> 4) == 5 && (ds3231.sec & 0b00001111) == 9) // min is 9, sec is 59
                    {
                        uint8_t buf_min = ((ds3231.min & 0b11110000) >> 4) + do_plus;
                        if (buf_min == 6)
                            buf_min = 0;
                        buf[0] = num_order[buf_min][0];
                        buf[1] = num_order[buf_min][1];
                    }
                    else
                    {
                        buf[0] = num_order[((ds3231.min & 0b11110000) >> 4)][0];
                        buf[1] = num_order[((ds3231.min & 0b11110000) >> 4)][1];
                    }
                }
                else if (tube_pos == 1)
                {
                    if (((ds3231.min & 0b11110000) >> 4) == 5 && (ds3231.min & 0b00001111) == 9 && ((ds3231.sec & 0b11110000) >> 4) == 5 && (ds3231.sec & 0b00001111) == 9) // min is 59, sec is 59
                    {
                        uint8_t buf_hour = (ds3231.hour & 0b00001111) + do_plus;
                        if (((ds3231.hour & 0b11110000) >> 4) != 2 && buf_hour == 10)
                            buf_hour = 0;
                        if (((ds3231.hour & 0b11110000) >> 4) == 2 && buf_hour == 4)
                            buf_hour = 0;
                        buf[0] = num_order[buf_hour][0];
                        buf[1] = num_order[buf_hour][1];
                    }
                    else
                    {
                        buf[0] = num_order[(ds3231.hour & 0b00001111)][0];
                        buf[1] = num_order[(ds3231.hour & 0b00001111)][1];
                    }
                }
                else if (tube_pos == 0)
                {
                    if (((ds3231.min & 0b11110000) >> 4) == 5 && (ds3231.min & 0b00001111) == 9 && ((ds3231.sec & 0b11110000) >> 4) == 5 && (ds3231.sec & 0b00001111) == 9) // min is 59, sec is 59
                    {
                        if (((ds3231.hour & 0b11110000) >> 4) == 2 && (ds3231.hour & 0b00001111) == 3) // hour is 23
                        {
                            uint8_t buf_hour = ((ds3231.hour & 0b11110000) >> 4) + do_plus;
                            if (buf_hour == 3)
                                buf_hour = 0;
                            buf[0] = num_order[buf_hour][0];
                            buf[1] = num_order[buf_hour][1];
                        }
                        else if (((ds3231.hour & 0b11110000) >> 4) != 2 && (ds3231.hour & 0b00001111) == 9) // hour is [0-1]9
                        {
                            uint8_t buf_hour = ((ds3231.hour & 0b11110000) >> 4) + do_plus;
                            buf[0] = num_order[buf_hour][0];
                            buf[1] = num_order[buf_hour][1];
                        }
                        else
                        {
                            buf[0] = num_order[(ds3231.hour & 0b11110000) >> 4][0];
                            buf[1] = num_order[(ds3231.hour & 0b11110000) >> 4][1];
                        }
                    }
                    else
                    {
                        buf[0] = num_order[(ds3231.hour & 0b11110000) >> 4][0];
                        buf[1] = num_order[(ds3231.hour & 0b11110000) >> 4][1];
                    }
                }
                else
                {
                    buf[0] = digit_num[tube_pos][0];
                    buf[1] = digit_num[tube_pos][1];
                }
                buf[2] = tube_num[tube_pos];
                gpio_put(NIXIE_LD_PIN, false);
                gpio_put(NIXIE_RD_PIN, false);
                // nixie_set_LD_pwm(0);
                // nixie_set_RD_pwm(0);
            }
            else
            {
                buf[0] = digit_num[tube_pos][0];
                buf[1] = digit_num[tube_pos][1];
                buf[2] = tube_num[tube_pos];
                gpio_put(NIXIE_LD_PIN, false);
                gpio_put(NIXIE_RD_PIN, false);
                // nixie_set_LD_pwm(0);
                // nixie_set_RD_pwm(0);
            }
        }
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    else if (duty_count >= setting.duty) // OFF
    {
        gpio_put(NIXIE_LD_PIN, false);
        gpio_put(NIXIE_RD_PIN, false);
        // nixie_set_LD_pwm(0);
        // nixie_set_RD_pwm(0);
        buf[0] = 0b00000000;
        buf[1] = 0b00000000;
        buf[2] = 0b11111111;
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    duty_count++;
    if (duty_count > DUTY_TUBE)
    {
        duty_count = 0;
        tube_pos++;
        if (tube_pos >= NIXIE_TUBE)
        {
            tube_pos = 0;
        }
    }
}

void pressure_mode(void)
{
    static int duty_count = 0;
    static int tube_pos = 0;
    uint8_t buf[3] = {0, 0, 0}; // SPI write buffer

    if (duty_count < setting.duty) // ON
    {
        if (tube_pos == 4)
        {
            dot_pattern();
            buf[0] = digit_num[tube_pos][0];
            buf[1] = digit_num[tube_pos][1];
            buf[2] = tube_num[tube_pos];
        }
        else
        {
            buf[0] = digit_num[tube_pos][0];
            buf[1] = digit_num[tube_pos][1];
            buf[2] = tube_num[tube_pos];
            gpio_put(NIXIE_LD_PIN, false);
            gpio_put(NIXIE_RD_PIN, false);
        }
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    else if (duty_count >= setting.duty) // OFF
    {
        gpio_put(NIXIE_LD_PIN, false);
        gpio_put(NIXIE_RD_PIN, false);
        buf[0] = 0b00000000;
        buf[1] = 0b00000000;
        buf[2] = 0b11111111;
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    duty_count++;
    if (duty_count > DUTY_TUBE)
    {
        duty_count = 0;
        tube_pos++;
        if (tube_pos >= NIXIE_TUBE)
        {
            tube_pos = 0;
        }
    }
}

void humidity_mode(void)
{
    static int duty_count = 0;
    static int tube_pos = 0;
    uint8_t buf[3] = {0, 0, 0}; // SPI write buffer

    if (duty_count < setting.duty) // ON
    {
        dot_pattern();
        buf[0] = digit_num[tube_pos][0];
        buf[1] = digit_num[tube_pos][1];
        buf[2] = tube_num[tube_pos];
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    else if (duty_count >= setting.duty) // OFF
    {
        gpio_put(NIXIE_LD_PIN, false);
        gpio_put(NIXIE_RD_PIN, false);
        buf[0] = 0b00000000;
        buf[1] = 0b00000000;
        buf[2] = 0b11111111;
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    duty_count++;
    if (duty_count > DUTY_TUBE)
    {
        duty_count = 0;
        tube_pos++;
        if (tube_pos >= NIXIE_TUBE)
        {
            tube_pos = 0;
        }
    }
}

void test_mode(void)
{
    static int duty_count = 0;
    static int tube_pos = 0;
    uint8_t buf[3] = {0, 0, 0}; // SPI write buffer

    if (duty_count < DUTY_MAX) // ON
    {
        dot_pattern();
        buf[0] = digit_num[tube_pos][0];
        buf[1] = digit_num[tube_pos][1];
        buf[2] = tube_num[tube_pos];
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    else if (duty_count >= DUTY_MAX) // OFF
    {
        gpio_put(NIXIE_LD_PIN, false);
        gpio_put(NIXIE_RD_PIN, false);
        buf[0] = 0b00000000;
        buf[1] = 0b00000000;
        buf[2] = 0b11111111;
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    duty_count++;
    if (duty_count > DUTY_TUBE)
    {
        duty_count = 0;
        tube_pos++;
        if (tube_pos >= NIXIE_TUBE)
        {
            tube_pos = 0;
        }
    }
}

// Nixie driver alarm 10us
bool alarm_nixie_driver_callback(struct repeating_timer *t)
{
    if (shuffle == false && anti_poisoning == false)
    {
        switch (mode)
        {
        case MODE_TIME:
            normal_mode();
            break;

        case MODE_DATE:
            normal_mode();
            break;

        //case MODE_HUMIDITY:
        //    humidity_mode();
        //    break;

        case MODE_TEST:
            test_mode();
            break;

        case MODE_PRESSURE:
            pressure_mode();
            break;

        default:
            normal_mode();
            break;
        }
    }
    else if (shuffle == true && anti_poisoning == false)
    {
        tube_shuffle();
    }
    else if (shuffle == false && anti_poisoning == true)
    {
        avoid_poisoning();
    }
    return true;
}