/**
 * Copyright (c) 2020 Raspberry Pi (Trading) Ltd.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
/*
// Remaining issues.
// HV DCDC ON/OFF 時間で
// dot pwm
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "hardware/adc.h"
#include "hardware/pwm.h"
#include "hardware/flash.h"
#include "hardware/sync.h"
#include "common.h"
#include "command.h"
#include "DS3231.h"
#include "Nixie_Driver.h"
#include "UART_GPS.h"
#include "shuffle.h"
#include "BME280.h"

// store RTC data. declared at DS3231.h
struct DS3231 ds3231 = {0};
// store following number SPI map for digits
uint8_t digit_num[8][2] = {0};
const uint8_t tube_num[8] = {
    // nixie tube digit SPI map
    0b11111110, // digit1 ON
    0b11111101, // digit2 ON
    0b11111011, // digit3 ON
    0b11110111, // digit4 ON
    0b11101111, // digit5 ON
    0b11011111, // digit6 ON
    0b10111111, // digit7 ON
    0b01111111  // digit8 ON
};
const uint8_t num_order[11][2] = {
    // nixie tube number SPI map
    {0b00000000, 0b00000001}, // cathode 0
    {0b00000000, 0b00000010}, // cathode 1
    {0b00000000, 0b00000100}, // cathode 2
    {0b00000000, 0b00001000}, // cathode 3
    {0b00000000, 0b00010000}, // cathode 4
    {0b00000000, 0b00100000}, // cathode 5
    {0b00000000, 0b01000000}, // cathode 6
    {0b00000001, 0b00000000}, // cathode 7
    {0b00000010, 0b00000000}, // cathode 8
    {0b00000100, 0b00000000}, // cathode 9
    {0b00000000, 0b00000000}  // suppress
};
// GPS data
bool gprmc_flag = false;
uint8_t gps_time[6] = {0};
uint8_t gps_date[6] = {0};
// W25Q16JV the last block(Block31) sector 0 address = 0x1F0000
const uint32_t flash_target_offset = 0x1F0000;
// Block30 sector 0 address = 0x1E0000
const uint32_t flash_calibration_data_offset = 0x1E0000;
// XIP_BASE(0x10000000) is defined at flash.h
// readable by XIP flash_setting_contents[0], flash_setting_contents[1]...
const uint8_t *flash_setting_contents = (const uint8_t *)(XIP_BASE + flash_target_offset);
const uint8_t *flash_calibration_data_contents = (const uint8_t *)(XIP_BASE + flash_calibration_data_offset);
// store QSPI flash config data. declared at common.h
struct SETTING setting = {0};
// store QSPI flash calibration data. declared at common.h
struct CALIBRATION calib = {0};
// data for write flash
// uint8_t write_data[FLASH_PAGE_SIZE];
// mode
int mode = 0;
int dot_mode = 0;
int led_mode = 0;
// shuffle
bool shuffle = false;
int X_num[NIXIE_TUBE] = {0};
bool X_pos[NIXIE_TUBE] = {0};
// xfade
int xfade_count = 0;
// avoid cathode poisoning
bool anti_poisoning = false;
int anti_posoning_count = 0;
// BME280
struct BME280_parameter BME280_param = {0};
int32_t temparature, pressure, humidity;

int main(void)
{
    // wait for start up
    sleep_ms(500);

    // read & check setting at QSPI flash
    check_setting();

    // UART & GPS
    init_uart();

    // I2C RTC DS3231M
    // init & attach callback(200ms)
    struct repeating_timer timer1;
    struct repeating_timer timer2;
    DS3231_init(&timer1, &timer2);

    // SPI Nixie Driver
    // init & attach callback(15us)
    struct repeating_timer timer0;
    SPI_init(&timer0);
    nixie_dot_init();

    // HV DCDC ON
    hv_dcdc_init();
    set_hv_dcdc(true);

    // I2C BME280
    BME280_init();
    BME280_read_all();

    // ADC init
    adc_init();

    // Switch & 1pps
    switch_init();

    // LED
    power_led_on();

    // read BME280 once
    BME280_read_all();

    // first random num
    int i, j;
    for (i = 0; i < adc_read_ADC0(); i++)
    {
        for (j = 0; j < NIXIE_TUBE; j++)
        {
            X_num[j] = rand_xor_shift() % 10; // 0~9
        }
    }

    // test mode
#ifndef NDEBUG
    mode = MODE_PRESSURE;
#endif

    while (true)
    {
    }
}