#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/gpio.h"
#include "hardware/uart.h"
#include "common.h"
#include "command.h"
#include "UART_GPS.h"
#include "extern.h"

// uart interrupt
void on_uart1_rx_tx(void)
{
    static char gps_buf[UART_BUF] = {0};
    static int gps_buf_pos = 0;

    if (uart_is_readable(UART_ID))
    {
        char ret = uart_getc(UART_ID);
        // buffer chars
        if (ret == '$')
        {
            gps_buf[0] = ret;
            gps_buf_pos = 1;
        }
        else
        {
            gps_buf[gps_buf_pos] = ret;
            gps_buf_pos++;
        }
        // when recieve '\n', recognize data.
        if (ret == '\n')
        {
            // for 1 sec increment, use other than 9 sec. for example use 1 sec(gps_buf[12] == '1').
            if (gps_buf[1] == 'G' && gps_buf[2] == 'P' && gps_buf[3] == 'R' && gps_buf[4] == 'M' && gps_buf[5] == 'C' && gps_buf[12] == '1' && gps_buf[18] == 'A')
            {
                // scan chars
                float gpstime, longitude, latitude, knot, angle;
                uint8_t date[6];
                sscanf(gps_buf, "$GPRMC,%f,A,%f,N,%f,E,%f,%f,%s", &gpstime, &longitude, &latitude, &knot, &angle, date);

                // hhmmss data ascii to decimal
                // instead of sprintf gpstime
                gps_time[0] = gps_buf[7] - 48;
                gps_time[1] = gps_buf[8] - 48;
                gps_time[2] = gps_buf[9] - 48;
                gps_time[3] = gps_buf[10] - 48;
                gps_time[4] = gps_buf[11] - 48;
                gps_time[5] = (gps_buf[12] - 48) + 1; // increase 1sec for 1pps update

                // ddmmyy data ascii to decimal
                gps_date[0] = date[0] - 48;
                gps_date[1] = date[1] - 48;
                gps_date[2] = date[2] - 48;
                gps_date[3] = date[3] - 48;
                gps_date[4] = date[4] - 48;
                gps_date[5] = date[5] - 48;

                // gprmc_flag is processed at GPS 1PPS callback
                if (setting.use_gps == true)
                {
                    gprmc_flag = true;
                    gps_sync_led_on();
                }
                else
                {
                    gprmc_flag = false;
                }
#ifndef NDEBUG
                printf("UTC TIME is %c%c/%c%c/%c%c %c%c:%c%c:%c%c\n", date[4], date[5], date[2], date[3], date[0], date[1], gps_buf[7], gps_buf[8], gps_buf[9], gps_buf[10], gps_buf[11], gps_buf[12]);
#endif
            }
        }
    }
}

void stdio_callback(void *param)
{
    static char input_buf[UART_BUF] = {0};
    static int input_pos = 0;
    char c;

    c = stdio_getchar();
    if (c != 0x7F) // without DEL
    {
        stdio_putchar_raw(c);
        input_buf[input_pos] = c;
        input_pos++;
        if (c == '\b') // BS
        {
            if(input_pos >= 2)
            {
                input_buf[input_pos] = 0;
                input_pos--; // Delete BS and one char
                input_pos--;
            }
            else
            {
                input_pos = 0;
            }
        }
    }
    if (c == '\r') // recieve enter
    {
        stdio_putchar_raw('\n'); // for terminal
        if (input_pos == 1) // just enter key
        {
            printf("WELCOME TO RP2040 NIXIE CLOCK.\n");
            printf("ENTER HELP COMMAND LIST.\n");
        }
        else
        {
            recognize_command(input_buf);
        }
        input_pos = 0;
        memset(input_buf, 0, sizeof(input_buf));
    }
}

void activate_gps(void)
{
    gpio_init(GPS_RESET_PIN);
    gpio_set_dir(GPS_RESET_PIN, GPIO_OUT);
    gpio_put(GPS_RESET_PIN, true);
}

void init_uart(void)
{
    // uart1 & GPS
    activate_gps();
    uart_init(UART_ID, BAUD_RATE);
    gpio_set_function(UART_TX_PIN, GPIO_FUNC_UART);
    gpio_set_function(UART_RX_PIN, GPIO_FUNC_UART);
    irq_set_exclusive_handler(UART1_IRQ, on_uart1_rx_tx);
    irq_set_enabled(UART1_IRQ, true);
    uart_set_irq_enables(UART_ID, true, true);

    // USB stdio
    stdio_init_all();
    stdio_set_chars_available_callback(stdio_callback, NULL);
}