#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hardware/gpio.h"
#include "hardware/flash.h"
#include "hardware/sync.h"
#include "Nixie_Driver.h"
#include "DS3231.h"
#include "common.h"
#include "command.h"
#include "UART_GPS.h"
#include "extern.h"

void recognize_command(char *buf)
{
    char str[UART_BUF];
    sscanf(buf, "%s", str); // remove return code and get only command

    if (strcmp(str, "HELP") == 0)
    {
        printf("%s yy/mm/dd/hh:mm:ss\n", SETTING_SET_DATE_STRING);
        printf("%s [%d to %d]\n", SETTING_DUTY_STRING, DUTY_DARK, DUTY_MAX);
        printf("%s [1/0]\n", SETTING_USE_SENSOR_STRING);
        printf("%s [1/0]\n", SETTING_USE_SENSOR_DARK_STRING);
        printf("%s [1/0]\n", SETTING_USE_GPS_STRING);
        printf("%s [1/0]\n", SETTING_USE_XFADE_STRING);
        printf("%s [%d to %d] (%d is the longest)\n", SETTING_USE_XFADE_LENGTH_STTRING, XFADE_MIN, XFADE_MAX, XFADE_MIN);
        printf("%s [1/0]\n", SETTING_USE_ANTI_POISONING_STRING);
        printf("%s hh:mm:ss\n", SETTING_ANTI_POISONING_TIME_STRING);
        printf("%s [%d to %d]\n", SETTING_TIMEZONE_STRING, TIME_ZONE_MIN, TIME_ZONE_MAX);
        printf("%s for shuffle.\n", SETTING_SHUFFLE_STRING);
        printf("SAVE to save setting.\n");
        printf("DEFAULT for default.\n");
    }
    else if (strcmp(str, SETTING_SET_DATE_STRING) == 0)
    {
        char temp_str[20], year_temp, mon_temp, date_temp, hour_temp, min_temp, sec_temp;
        sscanf(buf, "%s %hhu/%hhu/%hhu/%hhu:%hhu:%hhu", temp_str, &year_temp, &mon_temp, &date_temp, &hour_temp, &min_temp, &sec_temp);
        printf("Set %s %02hhu/%02hhu/%02hhu/%02hhu:%02hhu:%02hhu\n", temp_str, year_temp, mon_temp, date_temp, hour_temp, min_temp, sec_temp);
        DS3231_write_time_decimal(hour_temp, min_temp, sec_temp);
        DS3231_write_date_decimal(year_temp, mon_temp, date_temp);
    }
    else if (strcmp(str, SETTING_DUTY_STRING) == 0)
    {
        char temp_str[20], duty_temp;
        sscanf(buf, "%s %hhu", temp_str, &duty_temp);
        printf("Set %s %hhu\n", temp_str, duty_temp);
        setting.duty = duty_temp;
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_USE_SENSOR_STRING) == 0)
    {
        char temp_str[20], use_sensor_temp;
        sscanf(buf, "%s %hhu", temp_str, &use_sensor_temp);
        printf("Set %s %hhu\n", temp_str, use_sensor_temp);
        setting.use_sensor = use_sensor_temp;
        setting.duty = flash_setting_contents[SETTING_DUTY];
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_USE_SENSOR_DARK_STRING) == 0)
    {
        char temp_str[20], use_dark_temp;
        sscanf(buf, "%s %hhu", temp_str, &use_dark_temp);
        printf("Set %s %hhu\n", temp_str, use_dark_temp);
        setting.use_sensor_dark = use_dark_temp;
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_USE_GPS_STRING) == 0)
    {
        char temp_str[20], use_gps_temp;
        sscanf(buf, "%s %hhu", temp_str, &use_gps_temp);
        printf("Set %s %hhu\n", temp_str, use_gps_temp);
        setting.use_gps = use_gps_temp;
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_TIMEZONE_STRING) == 0)
    {
        char temp_str[20], timezone_temp;
        sscanf(buf, "%s %hhu", temp_str, &timezone_temp);
        printf("Set %s %hhu\n", temp_str, timezone_temp);
        setting.timezone = timezone_temp;
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_USE_XFADE_STRING) == 0)
    {
        char temp_str[20], use_xfade_temp;
        sscanf(buf, "%s %hhu", temp_str, &use_xfade_temp);
        printf("Set %s %hhu\n", temp_str, use_xfade_temp);
        setting.use_xfade = use_xfade_temp;
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_USE_XFADE_LENGTH_STTRING) == 0)
    {
        char temp_str[20], xfade_length_temp;
        sscanf(buf, "%s %hhu", temp_str, &xfade_length_temp);
        printf("Set %s %hhu\n", temp_str, xfade_length_temp);
        setting.xfade_length = xfade_length_temp;
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_USE_ANTI_POISONING_STRING) == 0)
    {
        char temp_str[20], use_anti_poisoning_temp;
        sscanf(buf, "%s %hhu", temp_str, &use_anti_poisoning_temp);
        printf("Set %s %hhu\n", temp_str, use_anti_poisoning_temp);
        setting.use_anti_poisoning = use_anti_poisoning_temp;
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_ANTI_POISONING_TIME_STRING) == 0)
    {
        char temp_str[20], hour_temp, min_temp, sec_temp;
        sscanf(buf, "%s %hhu:%hhu:%hhu", temp_str, &hour_temp, &min_temp, &sec_temp);
        printf("Set %s %02hhu:%02hhu:%02hhu\n", temp_str, hour_temp, min_temp, sec_temp);
        char temp_anti = 0;
        temp_anti = hour_temp / 10;
        setting.anti_poisoning_time[0] = (temp_anti << 4) + (hour_temp - temp_anti * 10);
        temp_anti = min_temp / 10;
        setting.anti_poisoning_time[1] = (temp_anti << 4) + (min_temp - temp_anti * 10);
        temp_anti = sec_temp / 10;
        setting.anti_poisoning_time[2] = (temp_anti << 4) + (sec_temp - temp_anti * 10);
        save_setting_to_flash();
    }
    else if (strcmp(str, SETTING_SHUFFLE_STRING) == 0)
    {
        shuffle = true;
    }
        else if (strcmp(str, CALIB_SENSOR_TH_STRING) == 0)
    {
        char temp_str[20], calibration_sensor;
        sscanf(buf, "%s %hhu", temp_str, &calibration_sensor);
        printf("Set %s %hhu\n", temp_str, calibration_sensor);
        calib.sensor_th = calibration_sensor;
        save_calibration_data_to_flash();
    }
    else if (strcmp(str, "SAVE") == 0)
    {
        printf("Save setting.\n");
        save_setting_to_flash();
        check_setting();
    }
    else if (strcmp(str, "DEFAULT") == 0)
    {
        printf("Default setting.\n");
        save_setting_to_flash_default();
        check_setting();
    }
    else if (strcmp(str, "CDD") == 0)
    {
        printf("Default calibration setting.\n");
        save_calibration_data_to_flash_default();
    }
    else
    {
        // printf("NG\n");
    }
}

void check_setting(void)
{
    bool write_flag = false;

    setting.duty = flash_setting_contents[SETTING_DUTY];
    setting.use_sensor = (bool)flash_setting_contents[SETTING_USE_SENSOR];
    setting.use_sensor_dark = (bool)flash_setting_contents[SETTING_USE_SENSOR_DARK];
    setting.use_gps = (bool)flash_setting_contents[SETTING_USE_GPS];
    setting.use_xfade = (bool)flash_setting_contents[SETTING_USE_XFADE];
    setting.xfade_length = flash_setting_contents[SETTING_XFADE_LENGTH];
    setting.timezone = (int8_t)flash_setting_contents[SETTING_TIMEZONE];
    setting.use_anti_poisoning = (bool)flash_setting_contents[SETTING_USE_ANTI_POISONING];
    setting.anti_poisoning_time[0] = flash_setting_contents[SETTING_ANTI_POISONING_TIME];
    setting.anti_poisoning_time[1] = flash_setting_contents[SETTING_ANTI_POISONING_TIME + 1];
    setting.anti_poisoning_time[2] = flash_setting_contents[SETTING_ANTI_POISONING_TIME + 2];
    setting.use_offtime = (bool)flash_setting_contents[SETTING_USE_OFFTIME];
    setting.offtime_start[0] = flash_setting_contents[SETTING_OFFTIME_START];
    setting.offtime_start[1] = flash_setting_contents[SETTING_OFFTIME_START + 1];
    setting.offtime_start[2] = flash_setting_contents[SETTING_OFFTIME_START + 2];
    setting.offtime_end[0] = flash_setting_contents[SETTING_OFFTIME_END];
    setting.offtime_end[1] = flash_setting_contents[SETTING_OFFTIME_END + 1];
    setting.offtime_end[2] = flash_setting_contents[SETTING_OFFTIME_END + 2];

    calib.sensor_th = flash_calibration_data_contents[CALIB_SENSOR_TH];

    if (setting.duty < DUTY_DARK || setting.duty > DUTY_MAX)
    {
        setting.duty = DUTY_NORMAL;
        write_flag = true;
    }
    if ((setting.use_sensor != true) && (setting.use_sensor != false))
    {
        setting.use_sensor = true;
        write_flag = true;
    }
    if ((setting.use_sensor_dark != true) && (setting.use_sensor_dark != false))
    {
        setting.use_sensor_dark = false;
        write_flag = true;
    }
    if ((setting.use_gps != true) && (setting.use_gps != false))
    {
        setting.use_gps = true;
        write_flag = true;
    }
    if ((setting.use_xfade != true) && (setting.use_xfade != false))
    {
        setting.use_xfade = true;
        write_flag = true;
    }
    if (setting.xfade_length > XFADE_MAX || setting.xfade_length < XFADE_MIN)
    {
        setting.xfade_length = XFADE_MIN;
        write_flag = true;
    }
    if (setting.timezone > TIME_ZONE_MAX || setting.timezone < TIME_ZONE_MIN)
    {
        setting.timezone = TIME_ZONE; // (+9)
        write_flag = true;
    }
    if (write_flag == true)
    {
        save_setting_to_flash();
    }
}

static void save_setting_to_flash(void)
{
    // FLASH_PAGE_SIZE(256Byte) is defined at flash.h
    uint8_t write_data[FLASH_PAGE_SIZE] = {0};

    // disable interrupts, and get flash flag
    uint32_t ints = save_and_disable_interrupts();

    write_data[SETTING_DUTY] = setting.duty;
    write_data[SETTING_USE_SENSOR] = setting.use_sensor;
    write_data[SETTING_USE_SENSOR_DARK] = setting.use_sensor_dark;
    write_data[SETTING_USE_GPS] = setting.use_gps;
    write_data[SETTING_TIMEZONE] = setting.timezone;
    write_data[SETTING_USE_XFADE] = setting.use_xfade;
    write_data[SETTING_XFADE_LENGTH] = setting.xfade_length;
    write_data[SETTING_USE_ANTI_POISONING] = setting.use_anti_poisoning;
    write_data[SETTING_ANTI_POISONING_TIME] = setting.anti_poisoning_time[0];
    write_data[SETTING_ANTI_POISONING_TIME + 1] = setting.anti_poisoning_time[1];
    write_data[SETTING_ANTI_POISONING_TIME + 2] = setting.anti_poisoning_time[2];
    write_data[SETTING_USE_OFFTIME] = 0;
    write_data[SETTING_OFFTIME_START] = 0;
    write_data[SETTING_OFFTIME_START + 1] = 0;
    write_data[SETTING_OFFTIME_START + 2] = 0;
    write_data[SETTING_OFFTIME_END] = 0;
    write_data[SETTING_OFFTIME_END + 1] = 0;
    write_data[SETTING_OFFTIME_END + 2] = 0;

    // erase flash
    // The erase unit is a multiple of FLASH_SECTOR_SIZE(4096Byte) defined in flash.h.
    flash_range_erase(flash_target_offset, FLASH_SECTOR_SIZE);
    // write flash
    // The write unit is a multiple of FLASH_PAGE_SIZE(256Byte) defined in flash.h.
    flash_range_program(flash_target_offset, write_data, FLASH_PAGE_SIZE);
    // restore flash flag
    restore_interrupts(ints);

    // return setting
    check_setting();
}

static void save_setting_to_flash_default(void)
{
    // FLASH_PAGE_SIZE(256Byte) is defined at flash.h
    uint8_t write_data[FLASH_PAGE_SIZE] = {0};

    // disable interrupts, and get flash flag
    uint32_t ints = save_and_disable_interrupts();

    write_data[SETTING_DUTY] = DUTY_NORMAL;
    write_data[SETTING_USE_SENSOR] = true;
    write_data[SETTING_USE_SENSOR_DARK] = false;
    write_data[SETTING_USE_GPS] = true;
    write_data[SETTING_TIMEZONE] = TIME_ZONE;
    write_data[SETTING_USE_XFADE] = true;
    write_data[SETTING_XFADE_LENGTH] = XFADE_NORMAL;
    write_data[SETTING_USE_ANTI_POISONING] = true;
    write_data[SETTING_ANTI_POISONING_TIME] = SETTING_ANTI_P_HOUR;
    write_data[SETTING_ANTI_POISONING_TIME + 1] = SETTING_ANTI_P_MIN;
    write_data[SETTING_ANTI_POISONING_TIME + 2] = SETTING_ANTI_P_SEC;
    write_data[SETTING_USE_OFFTIME] = 0;
    write_data[SETTING_OFFTIME_START] = 0;
    write_data[SETTING_OFFTIME_START + 1] = 0;
    write_data[SETTING_OFFTIME_START + 2] = 0;
    write_data[SETTING_OFFTIME_END] = 0;
    write_data[SETTING_OFFTIME_END + 1] = 0;
    write_data[SETTING_OFFTIME_END + 2] = 0;

    // erase flash
    // The erase unit is a multiple of FLASH_SECTOR_SIZE(4096Byte) defined in flash.h.
    flash_range_erase(flash_target_offset, FLASH_SECTOR_SIZE);
    // write flash
    // The write unit is a multiple of FLASH_PAGE_SIZE(256Byte) defined in flash.h.
    flash_range_program(flash_target_offset, write_data, FLASH_PAGE_SIZE);
    // restore flash flag
    restore_interrupts(ints);

    // return setting
    check_setting();
}

static void save_calibration_data_to_flash(void)
{
    // FLASH_PAGE_SIZE(256Byte) is defined at flash.h
    uint8_t write_data[FLASH_PAGE_SIZE] = {0};

    // disable interrupts, and get flash flag
    uint32_t ints = save_and_disable_interrupts();

    write_data[CALIB_SENSOR_TH] = calib.sensor_th;

    // erase flash
    // The erase unit is a multiple of FLASH_SECTOR_SIZE(4096Byte) defined in flash.h.
    flash_range_erase(flash_calibration_data_offset, FLASH_SECTOR_SIZE);
    // write flash
    // The write unit is a multiple of FLASH_PAGE_SIZE(256Byte) defined in flash.h.
    flash_range_program(flash_calibration_data_offset, write_data, FLASH_PAGE_SIZE);
    // restore flash flag
    restore_interrupts(ints);

    // return setting
    check_setting();
}

static void save_calibration_data_to_flash_default(void)
{
    // FLASH_PAGE_SIZE(256Byte) is defined at flash.h
    uint8_t write_data[FLASH_PAGE_SIZE] = {0};

    // disable interrupts, and get flash flag
    uint32_t ints = save_and_disable_interrupts();

    write_data[CALIB_SENSOR_TH] = ADC0_MIN;

    // erase flash
    // The erase unit is a multiple of FLASH_SECTOR_SIZE(4096Byte) defined in flash.h.
    flash_range_erase(flash_calibration_data_offset, FLASH_SECTOR_SIZE);
    // write flash
    // The write unit is a multiple of FLASH_PAGE_SIZE(256Byte) defined in flash.h.
    flash_range_program(flash_calibration_data_offset, write_data, FLASH_PAGE_SIZE);
    // restore flash flag
    restore_interrupts(ints);

    // return setting
    check_setting();
}
