#include <stdio.h>
#include <stdlib.h>
#include "hardware/gpio.h"
#include "hardware/adc.h"
#include "hardware/pwm.h"
#include "hardware/watchdog.h"
#include "DS3231.h"
#include "UART_GPS.h"
#include "Nixie_Driver.h"
#include "command.h"
#include "common.h"
#include "BME280.h"
#include "extern.h"

void digit_mode(void)
{
    switch (mode)
    {
    case MODE_TIME: // time
        digit_num[0][0] = num_order[(ds3231.hour & 0b11110000) >> 4][0];
        digit_num[0][1] = num_order[(ds3231.hour & 0b11110000) >> 4][1];
        digit_num[1][0] = num_order[(ds3231.hour & 0b00001111)][0];
        digit_num[1][1] = num_order[(ds3231.hour & 0b00001111)][1];
        digit_num[2][0] = num_order[10][0];
        digit_num[2][1] = num_order[10][1];
        digit_num[3][0] = num_order[(ds3231.min & 0b11110000) >> 4][0];
        digit_num[3][1] = num_order[(ds3231.min & 0b11110000) >> 4][1];
        digit_num[4][0] = num_order[(ds3231.min & 0b00001111)][0];
        digit_num[4][1] = num_order[(ds3231.min & 0b00001111)][1];
        digit_num[5][0] = num_order[10][0];
        digit_num[5][1] = num_order[10][1];
        digit_num[6][0] = num_order[(ds3231.sec & 0b11110000) >> 4][0];
        digit_num[6][1] = num_order[(ds3231.sec & 0b11110000) >> 4][1];
        digit_num[7][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[7][1] = num_order[(ds3231.sec & 0b00001111)][1];
        break;

    case MODE_DATE: // date
        digit_num[0][0] = num_order[(ds3231.year & 0b11110000) >> 4][0];
        digit_num[0][1] = num_order[(ds3231.year & 0b11110000) >> 4][1];
        digit_num[1][0] = num_order[(ds3231.year & 0b00001111)][0];
        digit_num[1][1] = num_order[(ds3231.year & 0b00001111)][1];
        digit_num[2][0] = num_order[10][0];
        digit_num[2][1] = num_order[10][1];
        digit_num[3][0] = num_order[(ds3231.mon & 0b11110000) >> 4][0];
        digit_num[3][1] = num_order[(ds3231.mon & 0b11110000) >> 4][1];
        digit_num[4][0] = num_order[(ds3231.mon & 0b00001111)][0];
        digit_num[4][1] = num_order[(ds3231.mon & 0b00001111)][1];
        digit_num[5][0] = num_order[10][0];
        digit_num[5][1] = num_order[10][1];
        digit_num[6][0] = num_order[(ds3231.date & 0b11110000) >> 4][0];
        digit_num[6][1] = num_order[(ds3231.date & 0b11110000) >> 4][1];
        digit_num[7][0] = num_order[(ds3231.date & 0b00001111)][0];
        digit_num[7][1] = num_order[(ds3231.date & 0b00001111)][1];
        break;

    case MODE_PRESSURE:
        char press_buf[NIXIE_TUBE + 1] = {0};
        sprintf(press_buf, "%08.3lf", (float)pressure / 100.0);
        digit_num[0][0] = num_order[press_buf[0] - 48][0];
        digit_num[0][1] = num_order[press_buf[0] - 48][1];
        digit_num[1][0] = num_order[press_buf[1] - 48][0];
        digit_num[1][1] = num_order[press_buf[1] - 48][1];
        digit_num[2][0] = num_order[press_buf[2] - 48][0];
        digit_num[2][1] = num_order[press_buf[2] - 48][1];
        digit_num[3][0] = num_order[press_buf[3] - 48][0];
        digit_num[3][1] = num_order[press_buf[3] - 48][1];
        digit_num[4][0] = num_order[10][0];
        digit_num[4][1] = num_order[10][1];
        digit_num[5][0] = num_order[press_buf[5] - 48][0];
        digit_num[5][1] = num_order[press_buf[5] - 48][1];
        digit_num[6][0] = num_order[press_buf[6] - 48][0];
        digit_num[6][1] = num_order[press_buf[6] - 48][1];
        digit_num[7][0] = num_order[press_buf[7] - 48][0];
        digit_num[7][1] = num_order[press_buf[7] - 48][1];
        break;

        /* omit temparature and humidity
            case MODE_HUMIDITY:
                char hum_buf[NIXIE_TUBE + 1] = {0};
                //sprintf(hum_buf, "%08.4lf", (float)humidity / 1024.0);
                sprintf(hum_buf, "%08.4lf", (float)temparature / 100.0); // sometimes humidity is broken.
                digit_num[0][0] = num_order[hum_buf[0] - 48][0];
                digit_num[0][1] = num_order[hum_buf[0] - 48][1];
                digit_num[1][0] = num_order[hum_buf[1] - 48][0];
                digit_num[1][1] = num_order[hum_buf[1] - 48][1];
                digit_num[2][0] = num_order[hum_buf[2] - 48][0];
                digit_num[2][1] = num_order[hum_buf[2] - 48][1];
                digit_num[3][0] = num_order[10][0];
                digit_num[3][1] = num_order[10][1];
                digit_num[4][0] = num_order[hum_buf[4] - 48][0];
                digit_num[4][1] = num_order[hum_buf[4] - 48][1];
                digit_num[5][0] = num_order[hum_buf[5] - 48][0];
                digit_num[5][1] = num_order[hum_buf[5] - 48][1];
                digit_num[6][0] = num_order[hum_buf[6] - 48][0];
                digit_num[6][1] = num_order[hum_buf[6] - 48][1];
                digit_num[7][0] = num_order[hum_buf[7] - 48][0];
                digit_num[7][1] = num_order[hum_buf[7] - 48][1];
                break;
        */
    case MODE_TEST:
        digit_num[0][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[0][1] = num_order[(ds3231.sec & 0b00001111)][1];
        digit_num[1][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[1][1] = num_order[(ds3231.sec & 0b00001111)][1];
        digit_num[2][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[2][1] = num_order[(ds3231.sec & 0b00001111)][1];
        digit_num[3][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[3][1] = num_order[(ds3231.sec & 0b00001111)][1];
        digit_num[4][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[4][1] = num_order[(ds3231.sec & 0b00001111)][1];
        digit_num[5][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[5][1] = num_order[(ds3231.sec & 0b00001111)][1];
        digit_num[6][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[6][1] = num_order[(ds3231.sec & 0b00001111)][1];
        digit_num[7][0] = num_order[(ds3231.sec & 0b00001111)][0];
        digit_num[7][1] = num_order[(ds3231.sec & 0b00001111)][1];
        break;

    default:
        mode = 0;
        break;
    }
}

void check_sensor(void)
{
    static int low_count = 0;
    static int high_count = 0;
    if (setting.use_sensor == true && mode != MODE_TEST)
    {
        unsigned short adc0_data = adc_read_ADC0();
        unsigned short adc1_data = adc_read_ADC1();
        if (setting.use_sensor_dark == false)
        {
            // if (adc0_data < ADC0_MIN && adc1_data < ADC0_MIN)
            if (adc0_data < calib.sensor_th && adc1_data < calib.sensor_th)
            {
                low_count++;
                if (low_count > ADC_COUNT)
                {
                    setting.duty = DUTY_DARK;
                    low_count--;
#ifndef NDEBUG
                    printf("ADC0=%d ADC1=%d TH=%d SET DARK\r\n", adc0_data, adc1_data, calib.sensor_th);
#endif
                }
                high_count--;
                if (high_count < 0)
                {
                    high_count = 0;
                }
            }
            // else if (adc0_data >= ADC0_MIN && adc1_data >= ADC0_MIN)
            else if (adc0_data >= calib.sensor_th && adc1_data >= calib.sensor_th)
            {
                high_count++;
                if (high_count > ADC_COUNT)
                {
                    setting.duty = flash_setting_contents[SETTING_DUTY];
                    high_count--;
#ifndef NDEBUG
                    printf("ADC0=%d ADC1=%d TH=%d SET BRIGHT\r\n", adc0_data, adc1_data, calib.sensor_th);
#endif
                }
                low_count--;
                if (low_count < 0)
                {
                    low_count = 0;
                }
            }
        }
        else
        {
            // if (adc0_data < ADC0_MIN && adc1_data < ADC0_MIN)
            if (adc0_data < calib.sensor_th && adc1_data < calib.sensor_th)
            {
                low_count++;
                if (low_count > ADC_COUNT)
                {
                    set_hv_dcdc(false);
                    low_count--;
#ifndef NDEBUG
                    printf("ADC0=%d ADC1=%d TH=%d OFF HV DCDC\r\n", adc0_data, adc1_data, calib.sensor_th);
#endif
                }
                high_count--;
                if (high_count < 0)
                {
                    high_count = 0;
                }
            }
            // else if (adc0_data >= ADC0_MIN && adc1_data >= ADC0_MIN)
            else if (adc0_data >= calib.sensor_th && adc1_data >= calib.sensor_th)
            {
                high_count++;
                if (high_count > ADC_COUNT)
                {
                    set_hv_dcdc(true);
                    high_count--;
#ifndef NDEBUG
                    printf("ADC0=%d ADC1=%d TH=%d ON HV DCDC\r\n", adc0_data, adc1_data, calib.sensor_th);
#endif
                }
                low_count--;
                if (low_count < 0)
                {
                    low_count = 0;
                }
            }
        }
    }
}

// 200ms alarm(update digit, ADC sensor, polling switch)
bool alarm_ms_callback(struct repeating_timer *t)
{
    // read RTC sec for dot sync
    DS3231_read_sec();

    // control digit
    digit_mode();

    // ADC sensor
    check_sensor();

    // switch polling
    static bool sw_old[3] = {0};
    static bool sw[3] = {0};
    bool sw_new[3] = {0};
    bool buf[3] = {0};

    sw_new[0] = gpio_get(SW0_PIN);
    sw_new[1] = gpio_get(SW1_PIN);
    sw_new[2] = gpio_get(SW2_PIN);
    buf[0] = ~(sw_old[0] ^ sw_new[0]) & (sw[0] ^ sw_new[0]); // twice same & afterward is different
    buf[1] = ~(sw_old[1] ^ sw_new[1]) & (sw[1] ^ sw_new[1]);
    buf[2] = ~(sw_old[2] ^ sw_new[2]) & (sw[2] ^ sw_new[2]);
    sw_old[0] = sw_new[0];
    sw_old[1] = sw_new[1];
    sw_old[2] = sw_new[2];
    sw[0] ^= buf[0];
    sw[1] ^= buf[1];
    sw[2] ^= buf[2];

    // sw stataus rise of fall
    if (sw[0] == true && sw_old[0] == false) // push
    {
#ifndef NDEBUG
        printf("SW0\r\n");
#endif
        mode++;
    }
    if (sw[1] == true && sw_old[1] == false) // left
    {
#ifndef NDEBUG
        printf("SW1\r\n");
#endif
        dot_mode++;
    }
    if (sw[2] == true && sw_old[2] == false) // right
    {
#ifndef NDEBUG
        printf("SW2\r\n");
#endif
        // led_mode++;
        shuffle = true;
        // anti_poisoning = true;
    }

    if (anti_poisoning == true)
    {
        anti_posoning_count++;
    }

    return true;
}

// for xfade 10ms
bool alarm_xfade_callback(struct repeating_timer *t)
{
    if (setting.use_xfade == true)
    {
        xfade_count++;
        if (xfade_count > 100)
            xfade_count = 0;
    }
    else
    {
        xfade_count = 0;
    }
    watchdog_update();
    return true;
}

void input_callback(uint gpio, uint32_t events)
{
    if (gpio == RTC_1PPS_PIN && events == GPIO_IRQ_EDGE_FALL)
    {
        rtc_1pps_led_toggle();
        DS3231_read_all();
        xfade_count = 0;
        //if (mode == MODE_HUMIDITY || mode == MODE_PRESSURE)
        if (mode == MODE_PRESSURE)
        {
            static int time = 0;
            time++;
            if (time > 3)
            {
                BME280_read_all();
                time = 0;
            }
        }
#ifndef NDEBUG
        printf("RTC 1pps\r\n");
#endif
    }
    if (gpio == GPS_1PPS_PIN && events == GPIO_IRQ_EDGE_RISE)
    {
        if (gprmc_flag == true)
        {
            // time
            DS3231_write_time(gps_time[4], gps_time[5], gps_time[2], gps_time[3], gps_time[0], gps_time[1]);
            // date
            DS3231_write_date(gps_date[0], gps_date[1], gps_date[2], gps_date[3], gps_date[4], gps_date[5]);
            gprmc_flag = false;
            gps_sync_led_off();
        }
#ifndef NDEBUG
        printf("GPS 1pps\r\n");
#endif
    }
}

void switch_init(void)
{
    gpio_init(SW0_PIN);
    gpio_init(SW1_PIN);
    gpio_init(SW2_PIN);
    gpio_init(GPS_1PPS_PIN);
    gpio_init(RTC_1PPS_PIN);
    gpio_set_dir(SW0_PIN, GPIO_IN);
    gpio_set_dir(SW1_PIN, GPIO_IN);
    gpio_set_dir(SW2_PIN, GPIO_IN);
    gpio_set_dir(GPS_1PPS_PIN, GPIO_IN);
    gpio_set_dir(RTC_1PPS_PIN, GPIO_IN);
    gpio_set_irq_enabled_with_callback(GPS_1PPS_PIN, GPIO_IRQ_EDGE_RISE, true, &input_callback);
    gpio_set_irq_enabled_with_callback(RTC_1PPS_PIN, GPIO_IRQ_EDGE_FALL, true, &input_callback);
}

void hv_dcdc_init(void)
{
    gpio_init(HV_DCDC_PIN);
    gpio_set_dir(HV_DCDC_PIN, GPIO_OUT);
}

void set_hv_dcdc(bool hv_dcdc_on)
{
    // high(true) is off, low(false) is on
    gpio_put(HV_DCDC_PIN, !hv_dcdc_on);
}

unsigned short adc_read_ADC0(void)
{
    adc_gpio_init(ADC0_PIN);
    adc_select_input(0);
    return adc_read();
}

unsigned short adc_read_ADC1(void)
{
    adc_gpio_init(ADC1_PIN);
    adc_select_input(1);
    return adc_read();
}

float adc_read_picotemp(void)
{
    adc_set_temp_sensor_enabled(true);
    adc_select_input(4);
    return 27 - ((adc_read() * 3.3f / 4095) - 0.706) / 0.001721;
}

// xor shift random
// 0 to 2^8 - 1
uint8_t rand_xor_shift(void)
{
    static uint8_t rand = XOR_SHIFT_SEED;
    static uint8_t count = 0;
    static uint8_t num = 0;
    static const uint8_t xor_shift_const[24][3] = {
        {1, 1, 2},
        {1, 1, 3},
        {1, 7, 3},
        {1, 7, 6},
        {1, 7, 7},
        {2, 1, 1},
        {2, 5, 5},
        {3, 1, 1},
        {3, 1, 5},
        {3, 5, 4},
        {3, 5, 5},
        {3, 5, 7},
        {3, 7, 1},
        {4, 5, 3},
        {5, 1, 3},
        {5, 3, 6},
        {5, 3, 7},
        {5, 5, 2},
        {5, 5, 3},
        {6, 3, 5},
        {6, 7, 1},
        {7, 3, 5},
        {7, 5, 3},
        {7, 7, 1}};

    rand ^= rand >> xor_shift_const[num][0];
    rand ^= rand << xor_shift_const[num][1];
    rand ^= rand >> xor_shift_const[num][2];

    // change const
    count++;
    if (count > 250)
    {
        num++;
    }
    if (num == 24)
    {
        num = 0;
    }
    return rand;
}

void power_led_on(void)
{
    gpio_init(POWER_LED);
    gpio_set_function(POWER_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(POWER_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_B, LED_POWER_PWM_NORMAL);
    pwm_set_enabled(slice_num, true);
}

void power_led_off(void)
{
    gpio_init(POWER_LED);
    gpio_set_function(POWER_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(POWER_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_B, 0);
    pwm_set_enabled(slice_num, true);
}

void power_led_toggle(void)
{
    int pwm = 0;
    static bool toggle = true;

    if (toggle == true)
    {
        toggle = false;
        pwm = LED_POWER_PWM_NORMAL;
    }
    else
    {
        toggle = true;
        pwm = 0;
    }

    gpio_init(POWER_LED);
    gpio_set_function(POWER_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(POWER_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_B, pwm);
    pwm_set_enabled(slice_num, true);
}

void rtc_1pps_led_on(void)
{
    gpio_init(RTC_1PPS_LED);
    gpio_set_function(RTC_1PPS_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(RTC_1PPS_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_B, LED_RTC_PWM_NORMAL);
    pwm_set_enabled(slice_num, true);
}

void rtc_1pps_led_off(void)
{
    gpio_init(RTC_1PPS_LED);
    gpio_set_function(RTC_1PPS_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(RTC_1PPS_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_B, 0);
    pwm_set_enabled(slice_num, true);
}

void rtc_1pps_led_toggle(void)
{
    int pwm = 0;
    static bool toggle = true;

    if (toggle == true)
    {
        toggle = false;
        pwm = LED_RTC_PWM_NORMAL;
    }
    else
    {
        toggle = true;
        pwm = 0;
    }

    gpio_init(RTC_1PPS_LED);
    gpio_set_function(RTC_1PPS_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(RTC_1PPS_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_B, pwm);
    pwm_set_enabled(slice_num, true);
}

void gps_sync_led_on(void)
{
    gpio_init(GPS_SYNC_LED);
    gpio_set_function(GPS_SYNC_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(GPS_SYNC_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_A, LED_GPS_PWM_NORMAL);
    pwm_set_enabled(slice_num, true);
}

void gps_sync_led_off(void)
{
    gpio_init(GPS_SYNC_LED);
    gpio_set_function(GPS_SYNC_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(GPS_SYNC_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_A, 0);
    pwm_set_enabled(slice_num, true);
}

void gps_sync_led_toggle(void)
{
    int pwm = 0;
    static bool toggle = true;

    if (toggle == true)
    {
        toggle = false;
        pwm = LED_GPS_PWM_NORMAL;
    }
    else
    {
        toggle = true;
        pwm = 0;
    }

    gpio_init(GPS_SYNC_LED);
    gpio_set_function(GPS_SYNC_LED, GPIO_FUNC_PWM);
    uint slice_num = pwm_gpio_to_slice_num(GPS_SYNC_LED);
    pwm_set_clkdiv(slice_num, LED_PWM_CLKDIV);
    pwm_set_wrap(slice_num, LED_PWM_WRAP);
    pwm_set_chan_level(slice_num, PWM_CHAN_A, pwm);
    pwm_set_enabled(slice_num, true);
}