#ifndef __COMMON_H
#define __COMMON_H

#include "pico/stdlib.h"

// switch PIN
#define SW0_PIN 18
#define SW1_PIN 19
#define SW2_PIN 20
// HV DCDC PIN
#define HV_DCDC_PIN 25
// LED PIN
#define LED0_PIN 21 // RED
#define LED1_PIN 16 // BLUE
#define LED2_PIN 17 // GREEN
#define LED_PWM_WRAP 256 // 8bit
#define LED_PWM_FREQ 1000 // 1kHz
#define LED_PWM_CLKDIV (float)(125000000 / ((LED_PWM_WRAP) * (LED_PWM_FREQ))) // 125000000 / (256 * 1000) = 4.8828125
#define LED0_PWM_MAX 100
#define LED0_PWM_MIN 0
#define LED_POWER_PWM_NORMAL 20
#define LED_RTC_PWM_NORMAL 256
#define LED_GPS_PWM_NORMAL 20
#define POWER_LED LED0_PIN
#define GPS_SYNC_LED LED1_PIN
#define RTC_1PPS_LED LED2_PIN

// ADC PIN
#define ADC0_PIN 26
#define ADC0_MIN 20
#define ADC1_PIN 27
#define ADC1_MIN ADC0_MIN
#define ADC_COUNT 5

// mode
#define MODE_TIME 0
#define MODE_DATE 1
#define MODE_PRESSURE 2
#define MODE_HUMIDITY 3
#define MODE_TEST MODE_HUMIDITY

// XOR shift seed
#define XOR_SHIFT_SEED 16

bool alarm_ms_callback(struct repeating_timer *t);
bool alarm_xfade_callback(struct repeating_timer *t);
bool alarm_switch_callback(struct repeating_timer *t);
void input_callback(uint gpio, uint32_t events);
void hv_dcdc_init(void);
void set_hv_dcdc(bool hv_dcdc_on);
void switch_init(void);
unsigned short adc_read_ADC0(void);
unsigned short adc_read_ADC1(void);
float adc_read_picotemp(void);
uint8_t rand_xor_shift(void);
void power_led_on(void);
void power_led_off(void);
void power_led_toggle(void);
void rtc_1pps_led_on(void);
void rtc_1pps_led_off(void);
void rtc_1pps_led_toggle(void);
void gps_sync_led_on(void);
void gps_sync_led_off(void);
void gps_sync_led_toggle(void);

struct SETTING
{
    uint8_t duty;
    bool use_sensor;
    bool use_sensor_dark;
    bool use_gps;
    bool use_xfade;
    uint8_t xfade_length;
    int8_t timezone;
    bool use_anti_poisoning;
    uint8_t anti_poisoning_time[3]; // hour, min, sec
    bool use_offtime;
    uint8_t offtime_start[3]; // hour, min, sec
    uint8_t offtime_end[3];   // hour, min, sec
};

struct CALIBRATION
{
    uint8_t sensor_th;
};

#endif