#include <stdio.h>
#include <stdlib.h>
#include "hardware/spi.h"
#include "common.h"
#include "Nixie_Driver.h"
#include "shuffle.h"
#include "extern.h"

static inline void cs_select(void)
{
    asm volatile("nop \n nop \n nop");
    gpio_put(PIN_CS, false); // Active low
    asm volatile("nop \n nop \n nop");
}

static inline void cs_deselect(void)
{
    asm volatile("nop \n nop \n nop");
    gpio_put(PIN_CS, true);
    asm volatile("nop \n nop \n nop");
}

void tube_shuffle(void)
{
    static int duty_count = 0;
    static int tube_pos = 0;
    static int phase_count = 0;
    static int phase = 0;
    uint8_t buf[3] = {0, 0, 0}; // SPI write buffer
    static int sub_count = 0;
    static uint8_t rand_num = 0;
    static uint8_t rand_num_count = 0;
    static bool rand_LD = true;
    static bool rand_RD = true;

    rand_num_count++;
    if (rand_num_count > SHUFFLE_FREQ)
    {
        rand_num_count = 0;
        rand_num = rand_xor_shift() % 10;
        rand_LD = rand_xor_shift() % 2;
        rand_RD = rand_xor_shift() % 2;
    }

    setting.use_sensor = false;
    setting.duty = DUTY_MAX;

    if (duty_count < setting.duty) // ON
    {
        switch (phase)
        {
        case 0: // shuffle
            //uint8_t rand_num = rand_xor_shift() % 10;
            //bool rand_LD = rand_xor_shift() % 2;
            //bool rand_RD = rand_xor_shift() % 2;
            buf[0] = num_order[rand_num][0];
            buf[1] = num_order[rand_num][1];
            buf[2] = tube_num[tube_pos];
            gpio_put(NIXIE_LD_PIN, rand_LD);
            gpio_put(NIXIE_RD_PIN, rand_RD);
            cs_select();
            spi_write_blocking(SPI_PORT, buf, 3);
            cs_deselect();
            break;

        case 1: // fix dot
            if (tube_pos != 1)
            {
                //uint8_t rand_num = rand_xor_shift() % 10;
                buf[0] = num_order[rand_num][0];
                buf[1] = num_order[rand_num][1];
                buf[2] = tube_num[tube_pos];
                gpio_put(NIXIE_LD_PIN, false);
                gpio_put(NIXIE_RD_PIN, false);
                cs_select();
                spi_write_blocking(SPI_PORT, buf, 3);
                cs_deselect();
            }
            else
            {
                buf[0] = num_order[10][0];
                buf[1] = num_order[10][1];
                buf[2] = tube_num[tube_pos];
                gpio_put(NIXIE_LD_PIN, false);
                gpio_put(NIXIE_RD_PIN, true);
                cs_select();
                spi_write_blocking(SPI_PORT, buf, 3);
                cs_deselect();
            }
            break;

        case 2: // random digit stop
            if (tube_pos != 1)
            {
                if (X_pos[tube_pos] == true)
                {
                    buf[0] = num_order[X_num[tube_pos]][0];
                    buf[1] = num_order[X_num[tube_pos]][1];
                    buf[2] = tube_num[tube_pos];
                    gpio_put(NIXIE_LD_PIN, false);
                    gpio_put(NIXIE_RD_PIN, false);
                    cs_select();
                    spi_write_blocking(SPI_PORT, buf, 3);
                    cs_deselect();
                }
                else
                {
                    //uint8_t rand_num = rand_xor_shift() % 10;
                    buf[0] = num_order[rand_num][0];
                    buf[1] = num_order[rand_num][1];
                    buf[2] = tube_num[tube_pos];
                    gpio_put(NIXIE_LD_PIN, false);
                    gpio_put(NIXIE_RD_PIN, false);
                    cs_select();
                    spi_write_blocking(SPI_PORT, buf, 3);
                    cs_deselect();
                }
                sub_count++;
                if (sub_count > PHASE_NUM_CHANGE)
                {
                    sub_count = 0;
                    X_pos[rand_xor_shift() % NIXIE_TUBE] = true;
                }
            }
            else
            {
                buf[0] = num_order[10][0];
                buf[1] = num_order[10][1];
                buf[2] = tube_num[tube_pos];
                gpio_put(NIXIE_LD_PIN, false);
                gpio_put(NIXIE_RD_PIN, true);
                cs_select();
                spi_write_blocking(SPI_PORT, buf, 3);
                cs_deselect();
            }
            break;

        case 3: // fix digit
            if (tube_pos != 1)
            {
                buf[0] = num_order[X_num[tube_pos]][0];
                buf[1] = num_order[X_num[tube_pos]][1];
                buf[2] = tube_num[tube_pos];
                gpio_put(NIXIE_LD_PIN, false);
                gpio_put(NIXIE_RD_PIN, false);
                cs_select();
                spi_write_blocking(SPI_PORT, buf, 3);
                cs_deselect();
            }
            else
            {
                buf[0] = num_order[10][0];
                buf[1] = num_order[10][1];
                buf[2] = tube_num[tube_pos];
                gpio_put(NIXIE_LD_PIN, false);
                gpio_put(NIXIE_RD_PIN, true);
                cs_select();
                spi_write_blocking(SPI_PORT, buf, 3);
                cs_deselect();
            }
            break;

        default:
            gpio_put(NIXIE_LD_PIN, false);
            gpio_put(NIXIE_RD_PIN, false);
            buf[0] = 0b00000000;
            buf[1] = 0b00000000;
            buf[2] = 0b11111111;
            cs_select();
            spi_write_blocking(SPI_PORT, buf, 3);
            cs_deselect();
            break;
        }
    }
    else if (duty_count >= setting.duty) // OFF
    {
        gpio_put(NIXIE_LD_PIN, false);
        gpio_put(NIXIE_RD_PIN, false);
        buf[0] = 0b00000000;
        buf[1] = 0b00000000;
        buf[2] = 0b11111111;
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    phase_count++;
    if (phase_count > PHASE_CHANGE)
    {
        phase++;
        phase_count = 0;
        if (phase == PHASE_LAST)
        {
            phase = 0;
            shuffle = false;
            setting.use_sensor = (bool)flash_setting_contents[SETTING_USE_SENSOR];
            setting.duty = flash_setting_contents[SETTING_DUTY];
            uint8_t i;
            for (i = 0; i < NIXIE_TUBE; i++)
            {
                X_num[i] = rand_xor_shift() % 10;
                X_pos[i] = false;
            }
        }
    }
    duty_count++;
    if (duty_count > DUTY_TUBE)
    {
        duty_count = 0;
        tube_pos++;
        if (tube_pos >= NIXIE_TUBE)
        {
            tube_pos = 0;
        }
    }
}

void avoid_poisoning(void)
{
    static int duty_count = 0;
    static int tube_pos = 0;
    uint8_t buf[3] = {0, 0, 0};  // SPI write buffer

    static uint8_t rand_num = 0;
    static uint8_t rand_num_count = 0;
    static bool rand_LD = true;
    static bool rand_RD = true;

    rand_num_count++;
    if (rand_num_count > SHUFFLE_FREQ)
    {
        rand_num_count = 0;
        rand_num = rand_xor_shift() % 10;
        rand_LD = rand_xor_shift() % 2;
        rand_RD = rand_xor_shift() % 2;
    }

    setting.use_sensor = false;
    setting.duty = DUTY_MAX;

    if (duty_count < setting.duty) // ON
    {
        buf[0] = num_order[rand_num][0];
        buf[1] = num_order[rand_num][1];
        buf[2] = tube_num[tube_pos];
        gpio_put(NIXIE_LD_PIN, rand_LD);
        gpio_put(NIXIE_RD_PIN, rand_RD);
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    else if (duty_count >= setting.duty) // OFF
    {
        gpio_put(NIXIE_LD_PIN, false);
        gpio_put(NIXIE_RD_PIN, false);
        buf[0] = 0b00000000;
        buf[1] = 0b00000000;
        buf[2] = 0b11111111;
        cs_select();
        spi_write_blocking(SPI_PORT, buf, 3);
        cs_deselect();
    }
    duty_count++;
    if (duty_count > DUTY_TUBE)
    {
        duty_count = 0;
        tube_pos++;
        if (tube_pos >= NIXIE_TUBE)
        {
            tube_pos = 0;
        }
    }
    if(anti_posoning_count > ANTI_POISONING_TIME_COUNT)
    {
        anti_poisoning = false;
        anti_posoning_count = 0;
        setting.use_sensor = (bool)flash_setting_contents[SETTING_USE_SENSOR];
        setting.duty = flash_setting_contents[SETTING_DUTY];
    }
}